/* test driver for wrpStack package */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>

#include "wrpstack.h"

#define STACK_QUIT	0
#define STACK_PUSH	1
#define STACK_POP		2
#define STACK_PEEK	3
#define STACK_LIST	4

char *Desc[] = {
"Quit program",	/* STACK_QUIT */
"Push value on top of stack",    /* STACK_PUSH */
"Pop value from top of stack",	/* STACK_POP */
"Look at top of Stack\n",			/* STACK_PEEK */
"List contents of stack\n",		/* STACK_LIST */
NULL};

struct cmd {
	char *str;	/* command string */
   int num;		/* command number */
};

static struct cmd Cmd[] = {
{ "push", STACK_PUSH},
{ "pop", STACK_POP},
{ "peek", STACK_PEEK},
{ "list", STACK_LIST},
{ ">", STACK_PUSH},
{ "<", STACK_POP},
{ "^", STACK_PEEK},
{ "!", STACK_LIST},
{NULL, NULL}};

void do_help(void);
void list_ent(int loc, void *ent);

int
main(int argc, char **argv)
{
	void * stack;
   char buf[BUFSIZ];
   int num, i;
   char *p, *str, *val;

   printf("Creating stack ...\n");
   stack = wrpStack_create();

   /* main loop - get command and do it */
	while (1)
   {
   	printf("%d items on stack.\n", wrpStack_count(stack));
		printf("Enter command, or ? for help: ");
      gets(buf);
      if (*buf == 'q') break;
      if (*buf == '?')
      {
      	do_help();
         continue;
      }

      /* a command and value should have been entered - parse string */
      for (p=buf; isspace(*p); p++);	/* find non-blank char */
      str = p;	/* this is the command string */
      while (*p != '\0' && !isspace(*p)) p++;	/* find end of string */
      if (*p != '\0') *p++ = '\0';	/* terminate command string */
      /* now skip past spaces to find start of value */
		while (*p != '\0' && isspace(*p)) p++;	/* find value */
      val = p;

      /* we now have a command string and a value - find match*/
		for (i=0; Cmd[i].str != NULL && strcmp(str, Cmd[i].str); i++);
      if (Cmd[i].str == NULL) printf("Command not found (%s)\n", str);
      else switch(Cmd[i].num)
      { /* process command */
      case STACK_PUSH:	/* push value */
      	/* we have to make a copy of what we push to the stack */
      	num = wrpStack_push(stack, strdup(val));	/* push value on stack */
         if (num != 0) printf("Stack error (%d)\n", num);
         break;

      case STACK_POP:	/* pop value from stack */
      	str = wrpStack_pop(stack);
         if (str == NULL) printf("Stack error\n");
         else
         {
         	printf("Stack returned >>%s<<\n", str);
            free(str);
         }
         break;

      case STACK_PEEK:	/* look at top of stack */
      	str = (char *)wrpStack_peek(stack);
         if (str == NULL) printf("Stack error\n");
         else printf("Top of Stack contains >>%s<<\n", str);
         break;

      case STACK_LIST:	/* list entire stack */
      	num = wrpStack_list(stack, list_ent);
         if (num != 0) printf("Stack error (%d)\n", num);
         break;

      default:
      	printf("Unknown command (%s)\n", str);
      }
   }

   printf("Freeing up stack ...\n");
   wrpStack_free(stack);

   printf("Press ENTER to continue: ");
   gets(buf);
   return 0;
}

void do_help(void)
{ /* display help message */
	int i;

	puts("This program is a test client for the wrpStack routines");
   puts("Enter a command and a value.  The value can be any string");
	puts("Valid commands are:");

   puts("Cmd\tDescription");
   for (i=0; Cmd[i].str != NULL; i++)
   { /* list commands */
   	printf("%s\t%s\n", Cmd[i].str, Desc[Cmd[i].num]);
   }
}

void list_ent(int loc, void *ent)
{
	char *str;

   if (ent == NULL) return;	/* bad arg */
   str = (char *)ent;
   printf("Entry %d >>%s<<\n", loc, str);
}

