/*
 *				**** WRP Stack ****
 *
 * This file contains an implementation of a stack using ProdList
 *
 */

#include <stdio.h>
#include <stdlib.h>

#include "prodlist.h"

#include "wrpstack.h"

static int debugsw = 0;	/* debugging switch */

void wrpStack_debug(int val) { debugsw = val; }

void *wrpStack_create(void)
{ /* create stack */
	ProdPtrList *ret;

   ret = create_ptr_prodlist(100);	/* create stack w/ space for 100 items */
   if (debugsw) printf("Stack now has %d items out of %d\n", ret->nitems, ret->maxitems);
   return (void *)ret;
}

void wrpStack_free(void * stack)
{ /* free up stack */
	if (stack == NULL) return;	/* bad args */
	free_ptr_prodlist((ProdPtrList *)stack);	/* free up list */
}

int wrpStack_push(void *stack, void *ent)
{ /* push entry onto top of stack */
	ProdPtrList *list, *newlist;

   if (stack == NULL || ent == NULL) return 1;	/* bad args */
   list = (ProdPtrList *)stack;	/* cast stack pointer to prodptrlist */
	if (list->nitems >= list->maxitems-1)
   { /* list full - expand */
		newlist = resize_ptr_prodlist(list, list->nitems*2);	/* double size */
		if (newlist == NULL) return 2;	/* out of memory */
   }
   list->item[(list->nitems)++] = ent;	/* add item */

   if (debugsw) printf("Stack now has %d items out of %d\n", list->nitems, list->maxitems);
   return 0;	/* normal return */
}

void * wrpStack_pop(void *stack)
{ /* pop entry from top of stack */
	ProdPtrList *list;
   void *ret;

   if (stack == NULL) return NULL; /* bad args */
   list = (ProdPtrList *)stack;

   if (list->nitems <= 0) return NULL;	/* nothing to pop */
   ret = list->item[--(list->nitems)];
   list->item[list->nitems] = NULL;	/* clear old entry */

   if (debugsw) printf("Stack now has %d items out of %d\n", list->nitems, list->maxitems);
   return ret;
}

void * wrpStack_peek(void *stack)
{ /* return top of stack without removing it */
	ProdPtrList *list;

	if (stack == NULL) return NULL;	/* bad args */
   list = (ProdPtrList *)stack;

   return list->item[list->nitems-1];
}

int wrpStack_list(void *stack, void (stacklist)(int loc, void *ent))
{
	int i;
   ProdPtrList *list;

   if (stack == NULL || stacklist == NULL) return 1; /* bad args */
   list = (ProdPtrList *)stack;

   for (i=0; i < list->nitems; i++)
   {
   	stacklist(i, list->item[i]);
   }
   return 0;
}

int wrpStack_count(void *stack)
{ /* return number of items on stack */
	ProdPtrList *list;

	if (stack == NULL) return -1;	/* bad arg */
   list = (ProdPtrList *)stack;
   return list->nitems;
}

